﻿using ProjectGasMachine.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGasMachine.Drawnings;

public class DrawningMachine
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityGas? EntityGas { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки газ автомобиля
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя координата прорисовки газ автомобиля
    /// </summary>
    protected int? _startPosY;
    /// <summary>
    /// Ширина прорисовки газ автомобиля
    /// </summary>
    private readonly int _drawningMachineWidth = 110;
    /// <summary>
    /// Высота прорисовки газ автомобиля
    /// </summary>
    private readonly int _drawningMachineHeight = 70;

    /// <summary>
    /// координата X объекта
    /// </summary>
    
    public int? GetPosX => _startPosX;

    /// <summary>
    /// координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// ширина объекта
    /// </summary>
    public int GetWidth => _drawningMachineWidth;
    /// <summary>
    /// высота объекта
    /// </summary>
    public int GetHeight => _drawningMachineHeight;




    /// <summary>
    /// пустой конструктор
    /// </summary>
    private DrawningMachine()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес машины</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawningMachine(int speed, double weight, Color bodyColor) : this()
    {
        EntityGas = new EntityGas(speed, weight, bodyColor);
       
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningMachineWidth">Ширина прорисовки газ автомобиля</param>
    /// <param name="drawningMachineHeight">Высота прорисовки газ автомобиля</param>

    protected DrawningMachine(int drawningMachineWidth, int drawningMachineHeight) : this()
    {
        _drawningMachineWidth = drawningMachineWidth;
        _pictureHeight = drawningMachineHeight;
    }
    /// <summary>
    /// Установка границ поля 
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих границах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (EntityGas == null)
        {
            return false;
        }

        if (width >= _drawningMachineWidth && height >= _drawningMachineHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;

            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                if (_startPosX + _drawningMachineWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningMachineWidth;
                }
                if (_startPosY + _drawningMachineHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningMachineHeight;
                }
            }
            return true;
        }
        return false;
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        _startPosX = x;
        _startPosY = y;
        if (x < 0)
        {
            _startPosX = 0;
        }

        if (y < 0)
        {
            _startPosY = 0;
        }

        if (x + _drawningMachineWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningMachineWidth;
        }

        if (y + _drawningMachineHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningMachineHeight;
        }
    }
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityGas == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            case DirectionType.Left:
                if (_startPosX.Value - EntityGas.Step > 0)
                {
                    _startPosX -= (int)EntityGas.Step;
                }
                return true;
            case DirectionType.Up:
                if (_startPosY.Value - EntityGas.Step > 0)
                {
                    _startPosY -= (int)EntityGas.Step;
                }
                return true;
            case DirectionType.Right:
                if (_startPosX.Value + EntityGas.Step + _drawningMachineWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityGas.Step;
                }
                return true;
            case DirectionType.Down:
                if (_startPosY.Value + EntityGas.Step + _drawningMachineHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityGas.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityGas == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);

       
        //границы автомобиля
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 40, 110, 10);
        g.DrawRectangle(pen, _startPosX.Value +  76, _startPosY.Value + 10, 30, 30);



        //кузов
        Brush br = new SolidBrush(EntityGas.BodyColor);
        g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 40, 110, 10);


        //кабина
        g.FillRectangle(br, _startPosX.Value + 76, _startPosY.Value + 10, 30, 30);


        

        //колеса


        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(br, _startPosX.Value + 10, _startPosY.Value + 50, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 85, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(br, _startPosX.Value + 85, _startPosY.Value + 50, 20, 20);





    }
}
